@@ -954,6 +954,23 @@ def test_visualize():
         visualize(x, filename=os.path.join(d, "mydask.png"))
         assert os.path.exists(os.path.join(d, "mydask.png"))
 
+        # To see if visualize() works when the filename parameter is set to None
+        # If the function raises an error, the test will fail
+        x.visualize(filename=None)
+
+
+@pytest.mark.skipif("not da")
+@pytest.mark.skipif(
+    sys.flags.optimize, reason="graphviz exception with Python -OO flag"
+)
+def test_visualize_highlevelgraph():
+    graphviz = pytest.importorskip("graphviz")
+    with tmpdir() as d:
+        x = da.arange(5, chunks=2)
+        viz = x.dask.visualize(filename=os.path.join(d, "mydask.png"))
+        # check visualization will automatically render in the jupyter notebook
+        assert isinstance(viz, graphviz.dot.Digraph)
+
 
 @pytest.mark.skipif(
     sys.flags.optimize, reason="graphviz exception with Python -OO flag"
